// -*- mode: c++; coding: utf-8 -*- 
/** vzproxyclient.h
 * VZ Proxy Client Library interface
 */
#ifndef __VZPROXYCLIENT_H
#define __VZPROXYCLIENT_H

#define  __STDC_LIMIT_MACROS
#include <stdint.h> 
#include <sys/types.h>
#include <errno.h> // for errno
#include <cstdio>  // for strerror()

#include <string>
#include <limits>
#include <exception>

#ifndef UINT32_MAX
static const u_int32_t UINT32_MAX = ~0;
#endif

namespace vz {

	class PacketGate {
	protected:
		PacketGate() {}; /// constructor is disabled from the outside
	public:
		static PacketGate* create(u_int32_t timeout = UINT32_MAX);
		virtual void send(const std::string& packet) = 0;
		virtual std::string recv() = 0;
		virtual ~PacketGate() = 0;
	};

	class EnvironmentError: public std::exception {
		std::string m_msg;
		void init(int _err) {
			_errno = _err;
			if (_errno) {
				m_msg += ": ";
				m_msg += ::strerror(_errno);
			}
		}

	public:
		int _errno;
		explicit
		EnvironmentError(const std::string& arg, int err):
			m_msg(arg) { init(err); };
		explicit
		EnvironmentError(const std::string& arg):
			m_msg(arg) { init(errno); };
		virtual ~EnvironmentError() throw() {};
		
		virtual const char* what() const throw() {
			try {
				return m_msg.c_str();
			} catch(...) { // Unlikely
				return strerror(_errno);
			}
		}
	};

	struct IOError: public EnvironmentError
	{
		IOError(const std::string& arg): EnvironmentError(arg) {};
		IOError(const std::string& arg, int err): EnvironmentError(arg, err) {};
	};
	
	struct OSError: public EnvironmentError
	{
		OSError(const std::string& arg): EnvironmentError(arg) {};
		OSError(const std::string& arg, int err): EnvironmentError(arg, err) {};
	};

}

#endif // __VZPROXYCLIENT_H
/*
 * Local Variables:      ***
 * c-file-style: "linux" ***
 * End:                  ***
 */
